﻿//============================================================================
/// \file   capi_dynload_test.cpp
/// \author Uwe Kindler
/// \date   12.06.2013
/// \brief  Implementation of 
//============================================================================


//============================================================================
//                                   INCLUDES
//============================================================================
#include <diag/diag.h>
#include <labbcan/c_api/labbCAN_Bus_API.h>
#include <conio.h>
#include <windows.h>
#include <iostream>



//============================================================================
//                              DEFINES
//============================================================================
#define _LOAD_FUNCTION_POINTER(DLL,FUNC) lp##FUNC = (PF_##FUNC)GetProcAddress(DLL, #FUNC);\
                                        lProcAddr&= lp##FUNC!=NULL?1:0;
#define UNMAP_FUNCTION_POINTER(FUNC)    lp##FUNC = 0
#define LOAD_FUNCTION_POINTER(FUNC)     _LOAD_FUNCTION_POINTER(st_hInstLib,FUNC)
#define DYNCALL(FUNC) lp##FUNC
#define DECLARE_FUNCTION_POINTER(FUNC)  PF_##FUNC lp##FUNC=NULL;


//============================================================================
//                             DATA TYPES
//============================================================================
typedef long (__stdcall* PF_LCB_Open) (const char* pDeviceConfigPath);
typedef long (__stdcall* PF_LCB_Close) ();


//============================================================================
//                            LOCAL DATA
//============================================================================
static HINSTANCE st_hInstLib;
static DWORD     st_hInstCounter = 0;
#define LCB_DLL_NAME L"labbCAN_Bus_APId.dll"

bool LCB_LoadLib(void);
bool LCB_FreeLib(void);
bool LCB_UnmapFunc(void);
bool LCB_UnmapFunc(void);

DECLARE_FUNCTION_POINTER(LCB_Open);
DECLARE_FUNCTION_POINTER(LCB_Close);

//============================================================================
// Load VCI2 DLL dynamically
//============================================================================
bool LCB_LoadLib(void)
{
    bool Result = false;

    if (!st_hInstCounter)
    {
        st_hInstLib = LoadLibrary(LCB_DLL_NAME);
        if (st_hInstLib)
        {
            st_hInstCounter = 1;
            Result = true;
        }
    }
    else
    {
        ++st_hInstCounter;
    }

    return Result;
}


//============================================================================
// Unload VCI2 DLL dynamically
//============================================================================
bool LCB_FreeLib(void)
{
    bool Result = false;

    if (!st_hInstCounter)
    {
        return true;
    }
    else if (1 == st_hInstCounter)
    {
        LCB_UnmapFunc();
        if (st_hInstLib)
        {
            FreeLibrary(st_hInstLib);
            st_hInstCounter = 0;
            Result = true;
        }
    }
    else
    {
        --st_hInstCounter;
    }

    return Result;
}


//============================================================================
// Sets function pointers to zero
//============================================================================
bool LCB_UnmapFunc(void)
{
    if (st_hInstCounter != 1)
    {
        return false;
    }

    if (st_hInstLib )
    {
        UNMAP_FUNCTION_POINTER(LCB_Open);
        UNMAP_FUNCTION_POINTER(LCB_Close);
    } // if (hInstCanLib )

    return true;
}


//============================================================================
// Initialize function pointers
//============================================================================
int LCB_MapFunc(void)
{
    int  Result     = 1;
    long lProcAddr  = 0x1;

    if (!st_hInstLib)
    {
        return 0;
    }

    LOAD_FUNCTION_POINTER(LCB_Open);
    LOAD_FUNCTION_POINTER(LCB_Close);

    //
    // mindestens eine Funktion wurde nicht gefunden
    //
    if(!lProcAddr)
    {
       Result = -1;
    }

    return Result;
}

int main(int argc, char **argv)
{
	std::cout << "Hello World!" << std::endl;
    if (!LCB_LoadLib())
    {
        std::cout << "ERR: CANO_EDLL_HW_NOT_FOUND - !LCB_LoadLib()" << std::endl;
        return -1;
    }

    int Result = LCB_MapFunc();
    if (1 == Result)
    {
        std::cout << "DLL dynamically loaded" << std::endl;
    }
    else if (0 == Result)
    {
        std::cout << "ERR: CANO_EDLL_HW_NOT_FOUND - LCB_MapFunc() returned %d" << std::endl;
        return -1;
    }
    else
    {
    	std::cout << "Invalid result" << std::endl;
        return -1;
    }

    lpLCB_Open("config/testconfig_nemesys");
    lpLCB_Close();

    LCB_UnmapFunc();
    if (!LCB_FreeLib())
    {
        std::cout << "ERR: CANO_EDLL_HWSWERR - !LCB_FreeLib()" << std::endl;
        return -1;
    }
    std::cout << "VCI3 DLL dynamically unloaded" << std::endl;
    std::cout << "Press any key..." << std::endl;
    int c = getch();

	return 0;
}


//---------------------------------------------------------------------------
// EOF capi_dynload_test.cpp
