# Generated by sila2.code_generator; sila2.__version__: 0.9.2
from __future__ import annotations

import math
from concurrent.futures import Executor
from threading import Event
from typing import TYPE_CHECKING, Dict

from sila2.framework.errors.validation_error import ValidationError
from sila2.server import MetadataDict

from sila_cetoni.heating_cooling.device_drivers.huber_chiller import HuberTemperatureControlMode

from ....device_drivers.abc import HuberChillerInterface
from ....device_drivers.abc import RangeLimit as DriverRangeLimit
from ..generated.huberchillercontrolservice import (
    ChangeSetPoint_Responses,
    ClearAlarmStatus_Responses,
    HuberChillerControlServiceBase,
    HuberChillerControlServiceFeature,
    RangeLimit,
    SetTemperatureControlMode_Responses,
    Temperature as TemperatureType,
    TemperatureControlMode as TemperatureControlModeType,
)

if TYPE_CHECKING:
    from ..server import Server


class HuberChillerControlServiceImpl(HuberChillerControlServiceBase):
    __temp_controller: HuberChillerInterface
    __stop_event: Event
    __executor: Executor
    __huber_temp_control_mode_to_user_friendly_string: Dict[HuberTemperatureControlMode, str] = {
        HuberTemperatureControlMode.INTERNAL: "On",
        HuberTemperatureControlMode.CIRCULATION: "Circulation Only",
        HuberTemperatureControlMode.OFF: "Off",
    }
    __user_friendly_string_to_huber_temp_control_mode: Dict[str, HuberTemperatureControlMode] = {
        v: k for k, v in __huber_temp_control_mode_to_user_friendly_string.items()
    }

    def __init__(self, parent_server: Server, temp_controller: HuberChillerInterface, executor: Executor):
        super().__init__(parent_server)
        self.__temp_controller = temp_controller
        self.__stop_event = Event()
        self.__executor = executor

        def update_alarm_status(stop_event: Event):
            new_status = status = self.__temp_controller.alarm_status
            while not stop_event.wait(0.1):
                new_status = self.__temp_controller.alarm_status
                if new_status != status:
                    status = new_status
                    self.update_AlarmStatus(status)

        def update_external_actual_value(stop_event: Event):
            new_value = value = self.__temp_controller.external_actual_value
            while not stop_event.wait(0.1):
                new_value = self.__temp_controller.external_actual_value
                if not math.isclose(new_value, value):
                    value = new_value
                    self.update_ExternalActualValue(value)

        def update_internal_actual_value(stop_event: Event):
            new_value = value = self.__temp_controller.internal_actual_value
            while not stop_event.wait(0.1):
                new_value = self.__temp_controller.internal_actual_value
                if not math.isclose(new_value, value):
                    value = new_value
                    self.update_InternalActualValue(value)

        def update_set_point_range_limit(stop_event: Event):
            new_limit = limit = self.__temp_controller.set_point_range_limit
            while not stop_event.wait(0.1):
                new_limit = self.__temp_controller.set_point_range_limit
                if new_limit != DriverRangeLimit() and (
                    not math.isclose(new_limit.min, limit.min) or not math.isclose(new_limit.max, limit.max)
                ):
                    limit = new_limit
                    self.update_SetPointRangeLimit(limit)

        def update_set_point_value(stop_event: Event):
            new_value = value = self.__temp_controller.set_point_value
            while not stop_event.wait(0.1):
                new_value = self.__temp_controller.set_point_value
                if not math.isclose(new_value, value):
                    value = new_value
                    self.update_SetPointValue(value)

        def update_temperature_control_mode(stop_event: Event):
            new_mode = mode = self.__temp_controller.temperature_control_mode
            while not stop_event.wait(0.1):
                new_mode = self.__temp_controller.temperature_control_mode
                if new_mode != mode:
                    mode = new_mode
                    self.update_TemperatureControlMode(self.__huber_temp_control_mode_to_user_friendly_string[mode])

        # initial values
        self.update_AlarmStatus(self.__temp_controller.alarm_status)
        self.update_ExternalActualValue(self.__temp_controller.external_actual_value)
        self.update_InternalActualValue(self.__temp_controller.internal_actual_value)
        self.update_SetPointRangeLimit(self.__temp_controller.set_point_range_limit)
        self.update_SetPointValue(self.__temp_controller.set_point_value)
        self.update_TemperatureControlMode(
            self.__huber_temp_control_mode_to_user_friendly_string[self.__temp_controller.temperature_control_mode]
        )

        executor.submit(update_alarm_status, self.__stop_event)
        executor.submit(update_external_actual_value, self.__stop_event)
        executor.submit(update_internal_actual_value, self.__stop_event)
        executor.submit(update_set_point_range_limit, self.__stop_event)
        executor.submit(update_set_point_value, self.__stop_event)
        executor.submit(update_temperature_control_mode, self.__stop_event)

    def get_OperatingRangeLimit(self, *, metadata: MetadataDict) -> RangeLimit:
        return self.__temp_controller.operating_range_limit

    def ChangeSetPoint(self, SetPointValue: TemperatureType, *, metadata: MetadataDict) -> ChangeSetPoint_Responses:
        min_set_point = self.__temp_controller.set_point_range_limit.min
        max_set_point = self.__temp_controller.set_point_range_limit.max
        if SetPointValue < min_set_point or SetPointValue > max_set_point:
            err = ValidationError(
                f"The given set point value {SetPointValue} is not within the allowed set point range "
                f"{(min_set_point, max_set_point)}"
            )
            err.parameter_fully_qualified_identifier = (
                HuberChillerControlServiceFeature["ChangeSetPoint"].parameters.fields[0].fully_qualified_identifier
            )
            raise err
        self.__executor.submit(setattr, self.__temp_controller, "set_point_value", SetPointValue)

    def SetTemperatureControlMode(
        self, TemperatureControlMode: TemperatureControlModeType, *, metadata: MetadataDict
    ) -> SetTemperatureControlMode_Responses:
        self.__executor.submit(
            setattr,
            self.__temp_controller,
            "temperature_control_mode",
            self.__user_friendly_string_to_huber_temp_control_mode[TemperatureControlMode],
        )

    def ClearAlarmStatus(self, *, metadata: MetadataDict) -> ClearAlarmStatus_Responses:
        self.__executor.submit(self.__temp_controller.clear_alarm_status)

    def stop(self) -> None:
        super().stop()
        self.__stop_event.set()
