# Generated by sila2.code_generator; sila2.__version__: 0.9.2
from __future__ import annotations

import logging
import math
import time
from concurrent.futures import Executor
from threading import Event
from typing import TYPE_CHECKING, Tuple

from sila2.framework.errors.validation_error import ValidationError
from sila2.server import MetadataDict

from sila_cetoni.heating_cooling.device_drivers.memmert_oven import EventLogEntry, EventLogEntryType
from sila_cetoni.heating_cooling.device_drivers.memmert_oven import (
    PermissionDenied as DriverPermissionDenied,
    ValueDidNotChange,
)

from ....device_drivers.abc import MemmertOvenInterface
from ....device_drivers.abc import Range as DriverRange
from ..generated.memmertovencontrolservice import (
    ChangeSetPoint_Responses,
    MemmertOvenControlServiceBase,
    Percent,
    PermissionDenied,
    ValueOutOfRange,
    Range,
    SetFanRotationSpeed_Responses,
    SetFlapPosition_Responses,
    SetMaximumAlarmTemperature_Responses,
    SetMinimumAlarmTemperature_Responses,
    Temperature,
)

if TYPE_CHECKING:
    from ..server import Server

logger = logging.getLogger(__name__)


class MemmertOvenControlServiceImpl(MemmertOvenControlServiceBase):
    __temp_controller: MemmertOvenInterface
    __stop_event: Event
    __executor: Executor

    def __init__(self, parent_server: Server, temp_controller: MemmertOvenInterface, executor: Executor) -> None:
        super().__init__(parent_server=parent_server)
        self.__temp_controller = temp_controller
        self.__stop_event = Event()
        self.__executor = executor

        def update_set_point_value(stop_event: Event):
            set_point_value = new_set_point_value = self.__temp_controller.set_point_value
            while not stop_event.wait(0.1):
                try:
                    new_set_point_value = self.__temp_controller.set_point_value
                    if not math.isclose(new_set_point_value, set_point_value):
                        set_point_value = new_set_point_value
                        self.update_SetPointValue(set_point_value)
                except DriverPermissionDenied as err:
                    self.abort_SetPointValue_subscriptions(self.make_permission_denied(err, "SetPointValue"))

        def update_actual_temperature(stop_event: Event):
            actual_temperature = new_actual_temperature = self.__temp_controller.actual_temperature
            while not stop_event.wait(0.1):
                try:
                    new_actual_temperature = self.__temp_controller.actual_temperature
                    if not math.isclose(new_actual_temperature, actual_temperature):
                        actual_temperature = new_actual_temperature
                        self.update_ActualTemperature(actual_temperature)
                except DriverPermissionDenied as err:
                    self.abort_ActualTemperature_subscriptions(self.make_permission_denied(err, "ActualTemperature"))

        def update_alarm_temperature(stop_event: Event):
            alarm_temperature = DriverRange(*self.__temp_controller.alarm_temperature)  # deep copy
            new_alarm_temperature = DriverRange(*alarm_temperature)  # deep copy
            while not stop_event.wait(0.1):
                try:
                    new_alarm_temperature = self.__temp_controller.alarm_temperature
                    if new_alarm_temperature != alarm_temperature:
                        alarm_temperature = DriverRange(*new_alarm_temperature)  # deep copy
                        self.update_AlarmTemperature(Range(*alarm_temperature))
                except DriverPermissionDenied as err:
                    self.abort_AlarmTemperature_subscriptions(self.make_permission_denied(err, "AlarmTemperature"))

        def update_fan_rotation_speed(stop_event: Event):
            fan_rotation_speed = new_fan_rotation_speed = self.__temp_controller.fan_rotation_speed
            while not stop_event.wait(0.1):
                try:
                    new_fan_rotation_speed = self.__temp_controller.fan_rotation_speed
                    if not math.isclose(new_fan_rotation_speed, fan_rotation_speed):
                        fan_rotation_speed = new_fan_rotation_speed
                        self.update_FanRotationSpeed(fan_rotation_speed)
                except DriverPermissionDenied as err:
                    self.abort_FanRotationSpeed_subscriptions(self.make_permission_denied(err, "FanRotationSpeed"))

        def update_flap_position(stop_event: Event):
            flap_position = new_flap_position = self.__temp_controller.flap_position
            while not stop_event.wait(0.1):
                try:
                    new_flap_position = self.__temp_controller.flap_position
                    if not math.isclose(new_flap_position, flap_position):
                        flap_position = new_flap_position
                        self.update_FlapPosition(flap_position)
                except DriverPermissionDenied as err:
                    self.abort_FlapPosition_subscriptions(self.make_permission_denied(err, "FlapPosition"))

        def update_min_temperature_underrun(stop_event: Event):
            min_temperature_underrun = new_min_temperature_underrun = self.__temp_controller.min_temperature_underrun
            while not stop_event.wait(0.1):
                new_min_temperature_underrun = self.__temp_controller.min_temperature_underrun
                if new_min_temperature_underrun != min_temperature_underrun:
                    min_temperature_underrun = new_min_temperature_underrun
                    self.update_MinimumTemperatureUnderrun(min_temperature_underrun)

        def update_max_temperature_exceeded(stop_event: Event):
            max_temperature_exceeded = new_max_temperature_exceeded = self.__temp_controller.max_temperature_exceeded
            while not stop_event.wait(0.1):
                new_max_temperature_exceeded = self.__temp_controller.max_temperature_exceeded
                if new_max_temperature_exceeded != max_temperature_exceeded:
                    max_temperature_exceeded = new_max_temperature_exceeded
                    self.update_MaximumTemperatureExceeded(max_temperature_exceeded)

        def update_event_log(stop_event: Event):
            event_log = new_event_log = self.__temp_controller.event_log
            while not stop_event.wait(0.1):
                new_event_log = self.__temp_controller.event_log
                if len(new_event_log) != len(event_log):
                    event_log = new_event_log
                    self.update_EventLog(map(self.convert_event_log_entry, event_log))

        # initial values
        self.update_SetPointValue(self.__temp_controller.set_point_value)
        self.update_ActualTemperature(self.__temp_controller.actual_temperature)
        self.update_AlarmTemperature(Range(*self.__temp_controller.alarm_temperature))
        self.update_FanRotationSpeed(self.__temp_controller.fan_rotation_speed)
        self.update_FlapPosition(self.__temp_controller.flap_position)
        self.update_MinimumTemperatureUnderrun(self.__temp_controller.min_temperature_underrun)
        self.update_MaximumTemperatureExceeded(self.__temp_controller.max_temperature_exceeded)
        self.update_EventLog(map(self.convert_event_log_entry, self.__temp_controller.event_log))

        self.__executor.submit(update_set_point_value, self.__stop_event)
        self.__executor.submit(update_actual_temperature, self.__stop_event)
        self.__executor.submit(update_alarm_temperature, self.__stop_event)
        self.__executor.submit(update_fan_rotation_speed, self.__stop_event)
        self.__executor.submit(update_flap_position, self.__stop_event)
        self.__executor.submit(update_min_temperature_underrun, self.__stop_event)
        self.__executor.submit(update_max_temperature_exceeded, self.__stop_event)
        self.__executor.submit(update_event_log, self.__stop_event)

    @staticmethod
    def make_permission_denied(err: DriverPermissionDenied, source: str) -> PermissionDenied:
        return PermissionDenied(
            f"The necessary permissions to access the {source} are missing. Required permissions: "
            f"{err.friendly_expected_permissions}, actual permissions: {err.friendly_actual_permissions}"
        )

    @staticmethod
    def make_value_out_of_range(err: ValueDidNotChange, source: str) -> ValueOutOfRange:
        return ValueOutOfRange(f"The value {err.value} is not in the valid range for the {source}")

    @staticmethod
    def convert_event_log_entry(entry: EventLogEntry) -> Tuple:
        description = entry.description
        if entry.entry_type == EventLogEntryType.INFORMATION:
            description = "Info: " + description
        else:
            description += " occurred" if entry.entry_type == EventLogEntryType.EVENT_OCCURRED else " finished"
        return (entry.code, description)

    def get_SetPointRange(self, *, metadata: MetadataDict) -> Range:
        try:
            return Range(*self.__temp_controller.set_point_range)
        except DriverPermissionDenied as err:
            raise self.make_permission_denied(err, "SetPointRange")

    def get_AlarmTemperatureRange(self, *, metadata: MetadataDict) -> Range:
        try:
            return Range(*self.__temp_controller.alarm_temperature_range)
        except DriverPermissionDenied as err:
            raise self.make_permission_denied(err, "AlarmTemperatureRange")

    def ChangeSetPoint(self, SetPointValue: Temperature, *, metadata: MetadataDict) -> ChangeSetPoint_Responses:
        try:
            self.__temp_controller.set_point_value = SetPointValue
        except DriverPermissionDenied as err:
            raise self.make_permission_denied(err, "SetPoint")
        except ValueDidNotChange as err:
            raise self.make_value_out_of_range(err, "SetPoint")

    def SetMinimumAlarmTemperature(
        self, MinimumAlarmTemperature: Temperature, *, metadata: MetadataDict
    ) -> SetMinimumAlarmTemperature_Responses:
        try:
            self.__temp_controller.set_alarm_temperature_min(MinimumAlarmTemperature)
        except DriverPermissionDenied as err:
            raise self.make_permission_denied(err, "MinimumAlarmTemperature")
        except ValueDidNotChange as err:
            raise self.make_value_out_of_range(err, "MinimumAlarmTemperature")

    def SetMaximumAlarmTemperature(
        self, MaximumAlarmTemperature: Temperature, *, metadata: MetadataDict
    ) -> SetMaximumAlarmTemperature_Responses:
        try:
            self.__temp_controller.set_alarm_temperature_max(MaximumAlarmTemperature)
        except DriverPermissionDenied as err:
            raise self.make_permission_denied(err, "MaximumAlarmTemperature")
        except ValueDidNotChange as err:
            raise self.make_value_out_of_range(err, "MaximumAlarmTemperature")

    def SetFanRotationSpeed(self, Speed: Percent, *, metadata: MetadataDict) -> SetFanRotationSpeed_Responses:
        try:
            self.__temp_controller.fan_rotation_speed = Speed
        except DriverPermissionDenied as err:
            raise self.make_permission_denied(err, "FanRotationSpeed")
        except ValueDidNotChange as err:
            raise self.make_value_out_of_range(err, "FanRotationSpeed")

    def SetFlapPosition(self, Position: Percent, *, metadata: MetadataDict) -> SetFlapPosition_Responses:
        try:
            self.__temp_controller.flap_position = Position
        except DriverPermissionDenied as err:
            raise self.make_permission_denied(err, "FlapPosition")
        except ValueDidNotChange as err:
            raise self.make_value_out_of_range(err, "FlapPosition")

    def stop(self) -> None:
        self.__stop_event.set()
        super().stop()
