# Generated by sila2.code_generator; sila2.__version__: 0.9.2
from __future__ import annotations

from abc import ABC, abstractmethod
from queue import Queue
from typing import TYPE_CHECKING, Any, List, Optional, Union

from sila2.server import FeatureImplementationBase, MetadataDict

from .memmertovencontrolservice_types import (
    ChangeSetPoint_Responses,
    Percent,
    SetFanRotationSpeed_Responses,
    SetFlapPosition_Responses,
    SetMaximumAlarmTemperature_Responses,
    SetMinimumAlarmTemperature_Responses,
    Temperature,
)

if TYPE_CHECKING:
    from ...server import Server


class MemmertOvenControlServiceBase(FeatureImplementationBase, ABC):
    parent_server: Server

    _SetPointValue_producer_queue: Queue[Union[Temperature, Exception]]

    _ActualTemperature_producer_queue: Queue[Union[Temperature, Exception]]

    _AlarmTemperature_producer_queue: Queue[Union[Range, Exception]]

    _FanRotationSpeed_producer_queue: Queue[Union[Percent, Exception]]

    _FlapPosition_producer_queue: Queue[Union[Percent, Exception]]

    _MinimumTemperatureUnderrun_producer_queue: Queue[Union[bool, Exception]]

    _MaximumTemperatureExceeded_producer_queue: Queue[Union[bool, Exception]]

    _EventLog_producer_queue: Queue[Union[List[Any], Exception]]

    def __init__(self, parent_server: Server):
        """
        Allows to control Memmert Oven devices
        """
        super().__init__(parent_server=parent_server)

        self._SetPointValue_producer_queue = Queue()

        self._ActualTemperature_producer_queue = Queue()

        self._AlarmTemperature_producer_queue = Queue()

        self._FanRotationSpeed_producer_queue = Queue()

        self._FlapPosition_producer_queue = Queue()

        self._MinimumTemperatureUnderrun_producer_queue = Queue()

        self._MaximumTemperatureExceeded_producer_queue = Queue()

        self._EventLog_producer_queue = Queue()

    @abstractmethod
    def get_SetPointRange(self, *, metadata: MetadataDict) -> Range:
        """
        The allowed range of temperatures for the Set Point Value

        :param metadata: The SiLA Client Metadata attached to the call
        :return: The allowed range of temperatures for the Set Point Value
        """
        pass

    @abstractmethod
    def get_AlarmTemperatureRange(self, *, metadata: MetadataDict) -> Range:
        """
        The allowed range of temperatures for the Minimum and Maximum Alarm Temperature

        :param metadata: The SiLA Client Metadata attached to the call
        :return: The allowed range of temperatures for the Minimum and Maximum Alarm Temperature
        """
        pass

    def update_SetPointValue(self, SetPointValue: Temperature, queue: Optional[Queue[Temperature]] = None) -> None:
        """
        The current set point value

        This method updates the observable property 'SetPointValue'.

        :param queue: The queue to send updates to. If None, the default Queue will be used.
        """
        if queue is None:
            queue = self._SetPointValue_producer_queue
        queue.put(SetPointValue)

    def SetPointValue_on_subscription(self, *, metadata: MetadataDict) -> Optional[Queue[Temperature]]:
        """
        The current set point value

        This method is called when a client subscribes to the observable property 'SetPointValue'

        :param metadata: The SiLA Client Metadata attached to the call
        :return: Optional `Queue` that should be used for updating this property.
            If None, the default Queue will be used.
        """
        pass

    def abort_SetPointValue_subscriptions(self, error: Exception, queue: Optional[Queue[Temperature]] = None) -> None:
        """
        The current set point value

        This method aborts subscriptions to the observable property 'SetPointValue'.

        :param error: The Exception to be sent to the subscribing client.
            If it is no DefinedExecutionError or UndefinedExecutionError, it will be wrapped in an UndefinedExecutionError.
        :param queue: The queue to abort. If None, the default Queue will be used.
        """
        if queue is None:
            queue = self._SetPointValue_producer_queue
        queue.put(error)

    def update_ActualTemperature(
        self, ActualTemperature: Temperature, queue: Optional[Queue[Temperature]] = None
    ) -> None:
        """
        The actual temperature of the device

        This method updates the observable property 'ActualTemperature'.

        :param queue: The queue to send updates to. If None, the default Queue will be used.
        """
        if queue is None:
            queue = self._ActualTemperature_producer_queue
        queue.put(ActualTemperature)

    def ActualTemperature_on_subscription(self, *, metadata: MetadataDict) -> Optional[Queue[Temperature]]:
        """
        The actual temperature of the device

        This method is called when a client subscribes to the observable property 'ActualTemperature'

        :param metadata: The SiLA Client Metadata attached to the call
        :return: Optional `Queue` that should be used for updating this property.
            If None, the default Queue will be used.
        """
        pass

    def abort_ActualTemperature_subscriptions(
        self, error: Exception, queue: Optional[Queue[Temperature]] = None
    ) -> None:
        """
        The actual temperature of the device

        This method aborts subscriptions to the observable property 'ActualTemperature'.

        :param error: The Exception to be sent to the subscribing client.
            If it is no DefinedExecutionError or UndefinedExecutionError, it will be wrapped in an UndefinedExecutionError.
        :param queue: The queue to abort. If None, the default Queue will be used.
        """
        if queue is None:
            queue = self._ActualTemperature_producer_queue
        queue.put(error)

    def update_AlarmTemperature(self, AlarmTemperature: Range, queue: Optional[Queue[Range]] = None) -> None:
        """
        The minimum and maximum alarm temperature values

        This method updates the observable property 'AlarmTemperature'.

        :param queue: The queue to send updates to. If None, the default Queue will be used.
        """
        if queue is None:
            queue = self._AlarmTemperature_producer_queue
        queue.put(AlarmTemperature)

    def AlarmTemperature_on_subscription(self, *, metadata: MetadataDict) -> Optional[Queue[Range]]:
        """
        The minimum and maximum alarm temperature values

        This method is called when a client subscribes to the observable property 'AlarmTemperature'

        :param metadata: The SiLA Client Metadata attached to the call
        :return: Optional `Queue` that should be used for updating this property.
            If None, the default Queue will be used.
        """
        pass

    def abort_AlarmTemperature_subscriptions(self, error: Exception, queue: Optional[Queue[Range]] = None) -> None:
        """
        The minimum and maximum alarm temperature values

        This method aborts subscriptions to the observable property 'AlarmTemperature'.

        :param error: The Exception to be sent to the subscribing client.
            If it is no DefinedExecutionError or UndefinedExecutionError, it will be wrapped in an UndefinedExecutionError.
        :param queue: The queue to abort. If None, the default Queue will be used.
        """
        if queue is None:
            queue = self._AlarmTemperature_producer_queue
        queue.put(error)

    def update_FanRotationSpeed(self, FanRotationSpeed: Percent, queue: Optional[Queue[Percent]] = None) -> None:
        """
        The rotation speed of the fan in percent

        This method updates the observable property 'FanRotationSpeed'.

        :param queue: The queue to send updates to. If None, the default Queue will be used.
        """
        if queue is None:
            queue = self._FanRotationSpeed_producer_queue
        queue.put(FanRotationSpeed)

    def FanRotationSpeed_on_subscription(self, *, metadata: MetadataDict) -> Optional[Queue[Percent]]:
        """
        The rotation speed of the fan in percent

        This method is called when a client subscribes to the observable property 'FanRotationSpeed'

        :param metadata: The SiLA Client Metadata attached to the call
        :return: Optional `Queue` that should be used for updating this property.
            If None, the default Queue will be used.
        """
        pass

    def abort_FanRotationSpeed_subscriptions(self, error: Exception, queue: Optional[Queue[Percent]] = None) -> None:
        """
        The rotation speed of the fan in percent

        This method aborts subscriptions to the observable property 'FanRotationSpeed'.

        :param error: The Exception to be sent to the subscribing client.
            If it is no DefinedExecutionError or UndefinedExecutionError, it will be wrapped in an UndefinedExecutionError.
        :param queue: The queue to abort. If None, the default Queue will be used.
        """
        if queue is None:
            queue = self._FanRotationSpeed_producer_queue
        queue.put(error)

    def update_FlapPosition(self, FlapPosition: Percent, queue: Optional[Queue[Percent]] = None) -> None:
        """
        The position of the flaps in percent (0 = fully closed, 100 = fully opened)

        This method updates the observable property 'FlapPosition'.

        :param queue: The queue to send updates to. If None, the default Queue will be used.
        """
        if queue is None:
            queue = self._FlapPosition_producer_queue
        queue.put(FlapPosition)

    def FlapPosition_on_subscription(self, *, metadata: MetadataDict) -> Optional[Queue[Percent]]:
        """
        The position of the flaps in percent (0 = fully closed, 100 = fully opened)

        This method is called when a client subscribes to the observable property 'FlapPosition'

        :param metadata: The SiLA Client Metadata attached to the call
        :return: Optional `Queue` that should be used for updating this property.
            If None, the default Queue will be used.
        """
        pass

    def abort_FlapPosition_subscriptions(self, error: Exception, queue: Optional[Queue[Percent]] = None) -> None:
        """
        The position of the flaps in percent (0 = fully closed, 100 = fully opened)

        This method aborts subscriptions to the observable property 'FlapPosition'.

        :param error: The Exception to be sent to the subscribing client.
            If it is no DefinedExecutionError or UndefinedExecutionError, it will be wrapped in an UndefinedExecutionError.
        :param queue: The queue to abort. If None, the default Queue will be used.
        """
        if queue is None:
            queue = self._FlapPosition_producer_queue
        queue.put(error)

    def update_MinimumTemperatureUnderrun(
        self, MinimumTemperatureUnderrun: bool, queue: Optional[Queue[bool]] = None
    ) -> None:
        """
        The current temperature is lower than the configured minimum alarm temperature

        This method updates the observable property 'MinimumTemperatureUnderrun'.

        :param queue: The queue to send updates to. If None, the default Queue will be used.
        """
        if queue is None:
            queue = self._MinimumTemperatureUnderrun_producer_queue
        queue.put(MinimumTemperatureUnderrun)

    def MinimumTemperatureUnderrun_on_subscription(self, *, metadata: MetadataDict) -> Optional[Queue[bool]]:
        """
        The current temperature is lower than the configured minimum alarm temperature

        This method is called when a client subscribes to the observable property 'MinimumTemperatureUnderrun'

        :param metadata: The SiLA Client Metadata attached to the call
        :return: Optional `Queue` that should be used for updating this property.
            If None, the default Queue will be used.
        """
        pass

    def abort_MinimumTemperatureUnderrun_subscriptions(
        self, error: Exception, queue: Optional[Queue[bool]] = None
    ) -> None:
        """
        The current temperature is lower than the configured minimum alarm temperature

        This method aborts subscriptions to the observable property 'MinimumTemperatureUnderrun'.

        :param error: The Exception to be sent to the subscribing client.
            If it is no DefinedExecutionError or UndefinedExecutionError, it will be wrapped in an UndefinedExecutionError.
        :param queue: The queue to abort. If None, the default Queue will be used.
        """
        if queue is None:
            queue = self._MinimumTemperatureUnderrun_producer_queue
        queue.put(error)

    def update_MaximumTemperatureExceeded(
        self, MaximumTemperatureExceeded: bool, queue: Optional[Queue[bool]] = None
    ) -> None:
        """
        The current temperature is higher than the configured maximum alarm temperature

        This method updates the observable property 'MaximumTemperatureExceeded'.

        :param queue: The queue to send updates to. If None, the default Queue will be used.
        """
        if queue is None:
            queue = self._MaximumTemperatureExceeded_producer_queue
        queue.put(MaximumTemperatureExceeded)

    def MaximumTemperatureExceeded_on_subscription(self, *, metadata: MetadataDict) -> Optional[Queue[bool]]:
        """
        The current temperature is higher than the configured maximum alarm temperature

        This method is called when a client subscribes to the observable property 'MaximumTemperatureExceeded'

        :param metadata: The SiLA Client Metadata attached to the call
        :return: Optional `Queue` that should be used for updating this property.
            If None, the default Queue will be used.
        """
        pass

    def abort_MaximumTemperatureExceeded_subscriptions(
        self, error: Exception, queue: Optional[Queue[bool]] = None
    ) -> None:
        """
        The current temperature is higher than the configured maximum alarm temperature

        This method aborts subscriptions to the observable property 'MaximumTemperatureExceeded'.

        :param error: The Exception to be sent to the subscribing client.
            If it is no DefinedExecutionError or UndefinedExecutionError, it will be wrapped in an UndefinedExecutionError.
        :param queue: The queue to abort. If None, the default Queue will be used.
        """
        if queue is None:
            queue = self._MaximumTemperatureExceeded_producer_queue
        queue.put(error)

    def update_EventLog(self, EventLog: List[Any], queue: Optional[Queue[List[Any]]] = None) -> None:
        """
        A list of events that were logged by the device

        This method updates the observable property 'EventLog'.

        :param queue: The queue to send updates to. If None, the default Queue will be used.
        """
        if queue is None:
            queue = self._EventLog_producer_queue
        queue.put(EventLog)

    def EventLog_on_subscription(self, *, metadata: MetadataDict) -> Optional[Queue[List[Any]]]:
        """
        A list of events that were logged by the device

        This method is called when a client subscribes to the observable property 'EventLog'

        :param metadata: The SiLA Client Metadata attached to the call
        :return: Optional `Queue` that should be used for updating this property.
            If None, the default Queue will be used.
        """
        pass

    def abort_EventLog_subscriptions(self, error: Exception, queue: Optional[Queue[List[Any]]] = None) -> None:
        """
        A list of events that were logged by the device

        This method aborts subscriptions to the observable property 'EventLog'.

        :param error: The Exception to be sent to the subscribing client.
            If it is no DefinedExecutionError or UndefinedExecutionError, it will be wrapped in an UndefinedExecutionError.
        :param queue: The queue to abort. If None, the default Queue will be used.
        """
        if queue is None:
            queue = self._EventLog_producer_queue
        queue.put(error)

    @abstractmethod
    def ChangeSetPoint(self, SetPointValue: Temperature, *, metadata: MetadataDict) -> ChangeSetPoint_Responses:
        """
        Change the set point value. This will start the temperature control if necessary.


        :param SetPointValue: The new set point value

        :param metadata: The SiLA Client Metadata attached to the call

        """
        pass

    @abstractmethod
    def SetMinimumAlarmTemperature(
        self, MinimumAlarmTemperature: Temperature, *, metadata: MetadataDict
    ) -> SetMinimumAlarmTemperature_Responses:
        """
        Set the minimum alarm temperature, i.e. the temperature that will cause an alarm if the actual temperature gets below it.


        :param MinimumAlarmTemperature: The new minimum alarm temperature

        :param metadata: The SiLA Client Metadata attached to the call

        """
        pass

    @abstractmethod
    def SetMaximumAlarmTemperature(
        self, MaximumAlarmTemperature: Temperature, *, metadata: MetadataDict
    ) -> SetMaximumAlarmTemperature_Responses:
        """
        Set the maximum alarm temperature, i.e. the temperature that will cause an alarm if the actual temperature gets above it.


        :param MaximumAlarmTemperature: The new maximum alarm temperature

        :param metadata: The SiLA Client Metadata attached to the call

        """
        pass

    @abstractmethod
    def SetFanRotationSpeed(self, Speed: Percent, *, metadata: MetadataDict) -> SetFanRotationSpeed_Responses:
        """
        Set the fan rotation speed


        :param Speed: The new fan rotation speed

        :param metadata: The SiLA Client Metadata attached to the call

        """
        pass

    @abstractmethod
    def SetFlapPosition(self, Position: Percent, *, metadata: MetadataDict) -> SetFlapPosition_Responses:
        """
        Set the flap position


        :param Position: The new flap position (0 = fully closed, 100 = fully opened)

        :param metadata: The SiLA Client Metadata attached to the call

        """
        pass
