# Generated by sila2.code_generator; sila2.__version__: 0.8.2

import logging
from typing import Optional, Union
from uuid import UUID

from sila2.server import SilaServer

from ...device_drivers.abc import HuberChillerInterface, MemmertOvenInterface, TemperatureControllerInterface
from .feature_implementations.huberchillercontrolservice_impl import HuberChillerControlServiceImpl
from .feature_implementations.memmertovencontrolservice_impl import MemmertOvenControlServiceImpl
from .generated.huberchillercontrolservice import HuberChillerControlServiceFeature
from .generated.memmertovencontrolservice import MemmertOvenControlServiceFeature

__version__ = "1.8.0"

logger = logging.getLogger(__name__)


class Server(SilaServer):
    def __init__(
        self,
        temp_controller: TemperatureControllerInterface,
        server_name: str = "",
        server_type: str = "",
        server_description: str = "",
        server_version: str = "",
        server_vendor_url: str = "",
        server_uuid: Optional[Union[str, UUID]] = None,
    ):
        super().__init__(
            server_name=server_name or "Heating/Cooling Service",
            server_type=server_type or "TestService",
            server_description=server_description or "The SiLA 2 driver for heating/cooling devices",
            server_version=server_version or __version__,
            server_vendor_url=server_vendor_url or "https://www.cetoni.com",
            server_uuid=server_uuid,
            max_child_task_workers=1000,
            max_grpc_workers=1000,
        )

        if isinstance(temp_controller, HuberChillerInterface):
            self.huberchillercontrolservice = HuberChillerControlServiceImpl(
                self, temp_controller, self.child_task_executor
            )
            self.set_feature_implementation(HuberChillerControlServiceFeature, self.huberchillercontrolservice)
        elif isinstance(temp_controller, MemmertOvenInterface):
            self.memmertovencontrolservice = MemmertOvenControlServiceImpl(
                self, temp_controller, self.child_task_executor
            )
            self.set_feature_implementation(MemmertOvenControlServiceFeature, self.memmertovencontrolservice)
        else:
            logger.warning(f"Unknown temperature controller interface type {temp_controller.__class__.__name__}")
