# Generated by sila2.code_generator; sila2.__version__: 0.8.0
from __future__ import annotations

from abc import ABC, abstractmethod
from queue import Queue
from typing import TYPE_CHECKING, List, Optional

from sila2.server import FeatureImplementationBase, MetadataDict

from .axissystemcontrolservice_types import (
    ClearFaultState_Responses,
    DisableAxisSystem_Responses,
    EnableAxisSystem_Responses,
)

if TYPE_CHECKING:
    from sila2.server import SilaServer


class AxisSystemControlServiceBase(FeatureImplementationBase, ABC):

    _AxisSystemState_producer_queue: Queue[str]

    _AxesInFaultState_producer_queue: Queue[List[str]]

    def __init__(self, parent_server: SilaServer):
        """
        Provides functionality to observe and control the state of an axis system
        """
        super().__init__(parent_server=parent_server)

        self._AxisSystemState_producer_queue = Queue()

        self._AxesInFaultState_producer_queue = Queue()

    @abstractmethod
    def get_AvailableAxes(self, *, metadata: MetadataDict) -> List[str]:
        """
        The names of the individual axes of the axis system.

        :param metadata: The SiLA Client Metadata attached to the call
        :return: The names of the individual axes of the axis system.
        """
        pass

    def update_AxisSystemState(self, AxisSystemState: str, queue: Optional[Queue[str]] = None):
        """
        The current state of the axis system. This is either 'Enabled' or 'Disabled'. Only if the state is 'Enabled', the axis system can move.

        This method updates the observable property 'AxisSystemState'.
        """
        if queue is None:
            queue = self._AxisSystemState_producer_queue
        queue.put(AxisSystemState)

    def AxisSystemState_on_subscription(self, *, metadata: MetadataDict) -> Optional[Queue[str]]:
        """
        The current state of the axis system. This is either 'Enabled' or 'Disabled'. Only if the state is 'Enabled', the axis system can move.

        This method is called when a client subscribes to the observable property 'AxisSystemState'

        :param metadata: The SiLA Client Metadata attached to the call
        :return: Optional `Queue` that should be used for updating this property.
            If None, the default Queue will be used.
        """
        pass

    def update_AxesInFaultState(self, AxesInFaultState: List[str], queue: Optional[Queue[List[str]]] = None):
        """
        Returns all axes of the system that are currently in fault state. The fault state of all axes can be cleared by calling ClearFaultState.

        This method updates the observable property 'AxesInFaultState'.
        """
        if queue is None:
            queue = self._AxesInFaultState_producer_queue
        queue.put(AxesInFaultState)

    def AxesInFaultState_on_subscription(self, *, metadata: MetadataDict) -> Optional[Queue[List[str]]]:
        """
        Returns all axes of the system that are currently in fault state. The fault state of all axes can be cleared by calling ClearFaultState.

        This method is called when a client subscribes to the observable property 'AxesInFaultState'

        :param metadata: The SiLA Client Metadata attached to the call
        :return: Optional `Queue` that should be used for updating this property.
            If None, the default Queue will be used.
        """
        pass

    @abstractmethod
    def EnableAxisSystem(self, *, metadata: MetadataDict) -> EnableAxisSystem_Responses:
        """
        Set all axes of the axis system into enabled state


        :param metadata: The SiLA Client Metadata attached to the call

        """
        pass

    @abstractmethod
    def DisableAxisSystem(self, *, metadata: MetadataDict) -> DisableAxisSystem_Responses:
        """
        Set all axes of the axis system into disabled state


        :param metadata: The SiLA Client Metadata attached to the call

        """
        pass

    @abstractmethod
    def ClearFaultState(self, *, metadata: MetadataDict) -> ClearFaultState_Responses:
        """
        Clears the fault condition of all axes. This is some kind of error acknowledge that clears the last fault and sets the device in an error-free state.


        :param metadata: The SiLA Client Metadata attached to the call

        """
        pass
