# Generated by sila2.code_generator; sila2.__version__: 0.8.0
from __future__ import annotations

from abc import ABC, abstractmethod
from queue import Queue
from typing import TYPE_CHECKING, Optional

from sila2.server import FeatureImplementationBase, MetadataDict

from .syringeconfigurationcontroller_types import SetSyringeParameters_Responses

if TYPE_CHECKING:
    from sila2.server import SilaServer


class SyringeConfigurationControllerBase(FeatureImplementationBase, ABC):

    _InnerDiameter_producer_queue: Queue[float]

    _MaxPistonStroke_producer_queue: Queue[float]

    def __init__(self, parent_server: SilaServer):
        """

        Provides syringe pump specific functions for configuration (i.e. the configuration of the syringe itself).

        """
        super().__init__(parent_server=parent_server)

        self._InnerDiameter_producer_queue = Queue()

        self._MaxPistonStroke_producer_queue = Queue()

    def update_InnerDiameter(self, InnerDiameter: float, queue: Optional[Queue[float]] = None):
        """
        Inner diameter of the syringe tube in millimetres.

        This method updates the observable property 'InnerDiameter'.
        """
        if queue is None:
            queue = self._InnerDiameter_producer_queue
        queue.put(InnerDiameter)

    def InnerDiameter_on_subscription(self, *, metadata: MetadataDict) -> Optional[Queue[float]]:
        """
        Inner diameter of the syringe tube in millimetres.

        This method is called when a client subscribes to the observable property 'InnerDiameter'

        :param metadata: The SiLA Client Metadata attached to the call
        :return: Optional `Queue` that should be used for updating this property.
            If None, the default Queue will be used.
        """
        pass

    def update_MaxPistonStroke(self, MaxPistonStroke: float, queue: Optional[Queue[float]] = None):
        """
        The maximum piston stroke defines the maximum position the piston can be moved to before it slips out of the syringe tube. The maximum piston stroke limits the maximum travel range of the syringe pump pusher.

        This method updates the observable property 'MaxPistonStroke'.
        """
        if queue is None:
            queue = self._MaxPistonStroke_producer_queue
        queue.put(MaxPistonStroke)

    def MaxPistonStroke_on_subscription(self, *, metadata: MetadataDict) -> Optional[Queue[float]]:
        """
        The maximum piston stroke defines the maximum position the piston can be moved to before it slips out of the syringe tube. The maximum piston stroke limits the maximum travel range of the syringe pump pusher.

        This method is called when a client subscribes to the observable property 'MaxPistonStroke'

        :param metadata: The SiLA Client Metadata attached to the call
        :return: Optional `Queue` that should be used for updating this property.
            If None, the default Queue will be used.
        """
        pass

    @abstractmethod
    def SetSyringeParameters(
        self, InnerDiameter: float, MaxPistonStroke: float, *, metadata: MetadataDict
    ) -> SetSyringeParameters_Responses:
        """

            Set syringe parameters.
            If you change the syringe in one device, you need to setup the new syringe parameters to get proper conversion of flow rate und volume units.



        :param InnerDiameter: Inner diameter of the syringe tube in millimetres.

        :param MaxPistonStroke: The maximum piston stroke defines the maximum position the piston can be moved to before it slips out of the syringe tube. The maximum piston stroke limits the maximum travel range of the syringe pump pusher.

        :param metadata: The SiLA Client Metadata attached to the call

        """
        pass
