# Generated by sila2.code_generator; sila2.__version__: 0.8.3
from __future__ import annotations

import datetime
import math
import time
from concurrent.futures import Executor
from threading import Event

from sila2.framework.data_types.date import SilaDateType
from sila2.server import MetadataDict, SilaServer

from ....device_drivers.abc import PurificationDeviceInterface
from ..generated.waterpurificationservice import (
    StartDispensing_Responses,
    StopDispensing_Responses,
    WaterPurificationServiceBase,
)

_LOCAL_TIMEZONE = datetime.datetime.now().astimezone().tzinfo


class WaterPurificationServiceImpl(WaterPurificationServiceBase):
    __device: PurificationDeviceInterface
    __executor: Executor
    __stop_event: Event

    def __init__(self, parent_server: SilaServer, device: PurificationDeviceInterface, executor: Executor):
        super().__init__(parent_server)
        self.__device = device
        self.__stop_event = Event()

        def update_intake_conductivity(stop_event: Event):
            new_intake_conductivity = intake_conductivity = self.__device.intake_conductivity
            while not stop_event.wait(0.1):
                new_intake_conductivity = self.__device.intake_conductivity
                if not math.isclose(new_intake_conductivity, intake_conductivity):
                    intake_conductivity = new_intake_conductivity
                    self.update_IntakeConductivity(intake_conductivity)

        def update_intake_temperature(stop_event: Event):
            new_intake_temperature = intake_temperature = self.__device.intake_temperature
            while not stop_event.wait(0.1):
                new_intake_temperature = self.__device.intake_temperature
                if not math.isclose(new_intake_temperature, intake_temperature):
                    intake_temperature = new_intake_temperature
                    self.update_IntakeTemperature(intake_temperature)

        def update_conductivity(stop_event: Event):
            new_conductivity = conductivity = self.__device.conductivity
            while not stop_event.wait(0.1):
                new_conductivity = self.__device.conductivity
                if not math.isclose(new_conductivity, conductivity):
                    conductivity = new_conductivity
                    self.update_Conductivity(conductivity)

        def update_temperature(stop_event: Event):
            new_temperature = temperature = self.__device.temperature
            while not stop_event.wait(0.1):
                new_temperature = self.__device.temperature
                if not math.isclose(new_temperature, temperature):
                    temperature = new_temperature
                    self.update_Temperature(temperature)

        def update_is_dispensing(stop_event: Event):
            new_is_dispensing = is_dispensing = self.__device.is_dispensing
            while not stop_event.wait(0.1):
                new_is_dispensing = self.__device.is_dispensing
                if new_is_dispensing != is_dispensing:
                    is_dispensing = new_is_dispensing
                    self.update_IsDispensing(is_dispensing)

        def update_is_dispensing_possible(stop_event: Event):
            new_is_dispensing_possible = is_dispensing_possible = self.__device.is_dispensing_possible
            while not stop_event.wait(0.1):
                new_is_dispensing_possible = self.__device.is_dispensing_possible
                if new_is_dispensing_possible != is_dispensing_possible:
                    is_dispensing_possible = new_is_dispensing_possible
                    self.update_IsDispensingPossible(is_dispensing_possible)

        def update_bag_tank_capacity(stop_event: Event):
            new_bag_tank_capacity = bag_tank_capacity = self.__device.bag_tank_capacity
            while not stop_event.wait(0.1):
                new_bag_tank_capacity = self.__device.bag_tank_capacity
                if not math.isclose(new_bag_tank_capacity, bag_tank_capacity):
                    bag_tank_capacity = new_bag_tank_capacity
                    self.update_BagTankCapacity(bag_tank_capacity)

        def update_bag_tank_available_volume(stop_event: Event):
            new_bag_tank_available_volume = bag_tank_available_volume = self.__device.bag_tank_available_volume
            while not stop_event.wait(0.1):
                new_bag_tank_available_volume = self.__device.bag_tank_available_volume
                if not math.isclose(new_bag_tank_available_volume, bag_tank_available_volume):
                    bag_tank_available_volume = new_bag_tank_available_volume
                    self.update_BagTankAvailableVolume(bag_tank_available_volume)

        def update_reminders(stop_event: Event):
            new_reminders = reminders = self.__device.reminders
            while not stop_event.wait(0.1):
                new_reminders = self.__device.reminders
                if len(new_reminders) != len(reminders):
                    reminders = new_reminders
                    self.update_Reminders([(r.name, SilaDateType(r.date, _LOCAL_TIMEZONE)) for r in reminders])

        def update_alerts(stop_event: Event):
            new_alerts = alerts = self.__device.alerts
            while not stop_event.wait(0.1):
                new_alerts = self.__device.alerts
                if len(new_alerts) != len(alerts):
                    alerts = new_alerts
                    self.update_Alerts(
                        [(a.level, a.category, a.issue, a.timestamp, a.acknowledged, a.muted) for a in alerts]
                    )

        def update_input_pressure(stop_event: Event):
            new_input_pressure = input_pressure = self.__device.input_pressure
            while not stop_event.wait(0.1):
                new_input_pressure = self.__device.input_pressure
                if not math.isclose(new_input_pressure, input_pressure):
                    input_pressure = new_input_pressure
                    self.update_InputPressure(input_pressure)

        def update_rejection_rate(stop_event: Event):
            new_rejection_rate = rejection_rate = self.__device.rejection_rate
            while not stop_event.wait(0.1):
                new_rejection_rate = self.__device.rejection_rate
                if not math.isclose(new_rejection_rate, rejection_rate):
                    rejection_rate = new_rejection_rate
                    self.update_RejectionRate(rejection_rate)

        self.update_IntakeConductivity(self.__device.intake_conductivity)
        self.update_IntakeTemperature(self.__device.intake_temperature)
        self.update_Conductivity(self.__device.conductivity)
        self.update_Temperature(self.__device.temperature)
        self.update_IsDispensing(self.__device.is_dispensing)
        self.update_IsDispensingPossible(self.__device.is_dispensing_possible)
        self.update_BagTankCapacity(self.__device.bag_tank_capacity)
        self.update_BagTankAvailableVolume(self.__device.bag_tank_available_volume)
        self.update_Reminders([(r.name, SilaDateType(r.date, _LOCAL_TIMEZONE)) for r in self.__device.reminders])
        self.update_Alerts(
            [(a.level, a.category, a.issue, a.timestamp, a.acknowledged, a.muted) for a in self.__device.alerts]
        )
        self.update_InputPressure(self.__device.input_pressure)
        self.update_RejectionRate(self.__device.rejection_rate)

        executor.submit(update_intake_conductivity, self.__stop_event)
        executor.submit(update_intake_temperature, self.__stop_event)
        executor.submit(update_conductivity, self.__stop_event)
        executor.submit(update_temperature, self.__stop_event)
        executor.submit(update_is_dispensing, self.__stop_event)
        executor.submit(update_is_dispensing_possible, self.__stop_event)
        executor.submit(update_bag_tank_capacity, self.__stop_event)
        executor.submit(update_bag_tank_available_volume, self.__stop_event)
        executor.submit(update_reminders, self.__stop_event)
        executor.submit(update_alerts, self.__stop_event)
        executor.submit(update_input_pressure, self.__stop_event)
        executor.submit(update_rejection_rate, self.__stop_event)

    def StartDispensing(self, *, metadata: MetadataDict) -> StartDispensing_Responses:
        self.__device.start_dispensing()

    def StopDispensing(self, *, metadata: MetadataDict) -> StopDispensing_Responses:
        self.__device.stop_dispensing()

    def stop(self) -> None:
        super().stop()
        self.__stop_event.set()
