# Generated by sila2.code_generator; sila2.__version__: 0.8.3
from __future__ import annotations

from abc import ABC, abstractmethod
from queue import Queue
from typing import TYPE_CHECKING, Any, List, Optional

from sila2.server import FeatureImplementationBase, MetadataDict

from .waterpurificationservice_types import StartDispensing_Responses, StopDispensing_Responses

if TYPE_CHECKING:
    from sila2.server import SilaServer


class WaterPurificationServiceBase(FeatureImplementationBase, ABC):

    _IntakeConductivity_producer_queue: Queue[Conductivity]

    _IntakeTemperature_producer_queue: Queue[Temperature]

    _InputPressure_producer_queue: Queue[Pressure]

    _Conductivity_producer_queue: Queue[Conductivity]

    _Temperature_producer_queue: Queue[Temperature]

    _RejectionRate_producer_queue: Queue[Percentage]

    _IsDispensing_producer_queue: Queue[bool]

    _IsDispensingPossible_producer_queue: Queue[bool]

    _BagTankCapacity_producer_queue: Queue[Litre]

    _BagTankAvailableVolume_producer_queue: Queue[Litre]

    _Reminders_producer_queue: Queue[List[Any]]

    _Alerts_producer_queue: Queue[List[Any]]

    def __init__(self, parent_server: SilaServer):
        """
        Allows to control water purification devices
        """
        super().__init__(parent_server=parent_server)

        self._IntakeConductivity_producer_queue = Queue()

        self._IntakeTemperature_producer_queue = Queue()

        self._InputPressure_producer_queue = Queue()

        self._Conductivity_producer_queue = Queue()

        self._Temperature_producer_queue = Queue()

        self._RejectionRate_producer_queue = Queue()

        self._IsDispensing_producer_queue = Queue()

        self._IsDispensingPossible_producer_queue = Queue()

        self._BagTankCapacity_producer_queue = Queue()

        self._BagTankAvailableVolume_producer_queue = Queue()

        self._Reminders_producer_queue = Queue()

        self._Alerts_producer_queue = Queue()

    def update_IntakeConductivity(self, IntakeConductivity: Conductivity, queue: Optional[Queue[Conductivity]] = None):
        """
        Conductivity of the water coming from the intake

        This method updates the observable property 'IntakeConductivity'.
        """
        if queue is None:
            queue = self._IntakeConductivity_producer_queue
        queue.put(IntakeConductivity)

    def IntakeConductivity_on_subscription(self, *, metadata: MetadataDict) -> Optional[Queue[Conductivity]]:
        """
        Conductivity of the water coming from the intake

        This method is called when a client subscribes to the observable property 'IntakeConductivity'

        :param metadata: The SiLA Client Metadata attached to the call
        :return: Optional `Queue` that should be used for updating this property.
            If None, the default Queue will be used.
        """
        pass

    def update_IntakeTemperature(self, IntakeTemperature: Temperature, queue: Optional[Queue[Temperature]] = None):
        """
        Temperature of the water coming from the intake

        This method updates the observable property 'IntakeTemperature'.
        """
        if queue is None:
            queue = self._IntakeTemperature_producer_queue
        queue.put(IntakeTemperature)

    def IntakeTemperature_on_subscription(self, *, metadata: MetadataDict) -> Optional[Queue[Temperature]]:
        """
        Temperature of the water coming from the intake

        This method is called when a client subscribes to the observable property 'IntakeTemperature'

        :param metadata: The SiLA Client Metadata attached to the call
        :return: Optional `Queue` that should be used for updating this property.
            If None, the default Queue will be used.
        """
        pass

    def update_InputPressure(self, InputPressure: Pressure, queue: Optional[Queue[Pressure]] = None):
        """
        The input pressure

        This method updates the observable property 'InputPressure'.
        """
        if queue is None:
            queue = self._InputPressure_producer_queue
        queue.put(InputPressure)

    def InputPressure_on_subscription(self, *, metadata: MetadataDict) -> Optional[Queue[Pressure]]:
        """
        The input pressure

        This method is called when a client subscribes to the observable property 'InputPressure'

        :param metadata: The SiLA Client Metadata attached to the call
        :return: Optional `Queue` that should be used for updating this property.
            If None, the default Queue will be used.
        """
        pass

    def update_Conductivity(self, Conductivity: Conductivity, queue: Optional[Queue[Conductivity]] = None):
        """
        Conductivity of the purified water

        This method updates the observable property 'Conductivity'.
        """
        if queue is None:
            queue = self._Conductivity_producer_queue
        queue.put(Conductivity)

    def Conductivity_on_subscription(self, *, metadata: MetadataDict) -> Optional[Queue[Conductivity]]:
        """
        Conductivity of the purified water

        This method is called when a client subscribes to the observable property 'Conductivity'

        :param metadata: The SiLA Client Metadata attached to the call
        :return: Optional `Queue` that should be used for updating this property.
            If None, the default Queue will be used.
        """
        pass

    def update_Temperature(self, Temperature: Temperature, queue: Optional[Queue[Temperature]] = None):
        """
        Temperature of the purified water

        This method updates the observable property 'Temperature'.
        """
        if queue is None:
            queue = self._Temperature_producer_queue
        queue.put(Temperature)

    def Temperature_on_subscription(self, *, metadata: MetadataDict) -> Optional[Queue[Temperature]]:
        """
        Temperature of the purified water

        This method is called when a client subscribes to the observable property 'Temperature'

        :param metadata: The SiLA Client Metadata attached to the call
        :return: Optional `Queue` that should be used for updating this property.
            If None, the default Queue will be used.
        """
        pass

    def update_RejectionRate(self, RejectionRate: Percentage, queue: Optional[Queue[Percentage]] = None):
        """
        The percentage of water that is rejected during the purification process

        This method updates the observable property 'RejectionRate'.
        """
        if queue is None:
            queue = self._RejectionRate_producer_queue
        queue.put(RejectionRate)

    def RejectionRate_on_subscription(self, *, metadata: MetadataDict) -> Optional[Queue[Percentage]]:
        """
        The percentage of water that is rejected during the purification process

        This method is called when a client subscribes to the observable property 'RejectionRate'

        :param metadata: The SiLA Client Metadata attached to the call
        :return: Optional `Queue` that should be used for updating this property.
            If None, the default Queue will be used.
        """
        pass

    def update_IsDispensing(self, IsDispensing: bool, queue: Optional[Queue[bool]] = None):
        """
        Whether or not the device is currently dispensing purified water

        This method updates the observable property 'IsDispensing'.
        """
        if queue is None:
            queue = self._IsDispensing_producer_queue
        queue.put(IsDispensing)

    def IsDispensing_on_subscription(self, *, metadata: MetadataDict) -> Optional[Queue[bool]]:
        """
        Whether or not the device is currently dispensing purified water

        This method is called when a client subscribes to the observable property 'IsDispensing'

        :param metadata: The SiLA Client Metadata attached to the call
        :return: Optional `Queue` that should be used for updating this property.
            If None, the default Queue will be used.
        """
        pass

    def update_IsDispensingPossible(self, IsDispensingPossible: bool, queue: Optional[Queue[bool]] = None):
        """
        Whether or not it is currently possible to dispense purified water

        This method updates the observable property 'IsDispensingPossible'.
        """
        if queue is None:
            queue = self._IsDispensingPossible_producer_queue
        queue.put(IsDispensingPossible)

    def IsDispensingPossible_on_subscription(self, *, metadata: MetadataDict) -> Optional[Queue[bool]]:
        """
        Whether or not it is currently possible to dispense purified water

        This method is called when a client subscribes to the observable property 'IsDispensingPossible'

        :param metadata: The SiLA Client Metadata attached to the call
        :return: Optional `Queue` that should be used for updating this property.
            If None, the default Queue will be used.
        """
        pass

    def update_BagTankCapacity(self, BagTankCapacity: Litre, queue: Optional[Queue[Litre]] = None):
        """
        Total capacity of the bag tank holding the purified water

        This method updates the observable property 'BagTankCapacity'.
        """
        if queue is None:
            queue = self._BagTankCapacity_producer_queue
        queue.put(BagTankCapacity)

    def BagTankCapacity_on_subscription(self, *, metadata: MetadataDict) -> Optional[Queue[Litre]]:
        """
        Total capacity of the bag tank holding the purified water

        This method is called when a client subscribes to the observable property 'BagTankCapacity'

        :param metadata: The SiLA Client Metadata attached to the call
        :return: Optional `Queue` that should be used for updating this property.
            If None, the default Queue will be used.
        """
        pass

    def update_BagTankAvailableVolume(self, BagTankAvailableVolume: Litre, queue: Optional[Queue[Litre]] = None):
        """
        Available volume of purified water in the bag tank

        This method updates the observable property 'BagTankAvailableVolume'.
        """
        if queue is None:
            queue = self._BagTankAvailableVolume_producer_queue
        queue.put(BagTankAvailableVolume)

    def BagTankAvailableVolume_on_subscription(self, *, metadata: MetadataDict) -> Optional[Queue[Litre]]:
        """
        Available volume of purified water in the bag tank

        This method is called when a client subscribes to the observable property 'BagTankAvailableVolume'

        :param metadata: The SiLA Client Metadata attached to the call
        :return: Optional `Queue` that should be used for updating this property.
            If None, the default Queue will be used.
        """
        pass

    def update_Reminders(self, Reminders: List[Any], queue: Optional[Queue[List[Any]]] = None):
        """
        A list of upcoming reminders

        This method updates the observable property 'Reminders'.
        """
        if queue is None:
            queue = self._Reminders_producer_queue
        queue.put(Reminders)

    def Reminders_on_subscription(self, *, metadata: MetadataDict) -> Optional[Queue[List[Any]]]:
        """
        A list of upcoming reminders

        This method is called when a client subscribes to the observable property 'Reminders'

        :param metadata: The SiLA Client Metadata attached to the call
        :return: Optional `Queue` that should be used for updating this property.
            If None, the default Queue will be used.
        """
        pass

    def update_Alerts(self, Alerts: List[Any], queue: Optional[Queue[List[Any]]] = None):
        """
        A list of the currently active alerts

        This method updates the observable property 'Alerts'.
        """
        if queue is None:
            queue = self._Alerts_producer_queue
        queue.put(Alerts)

    def Alerts_on_subscription(self, *, metadata: MetadataDict) -> Optional[Queue[List[Any]]]:
        """
        A list of the currently active alerts

        This method is called when a client subscribes to the observable property 'Alerts'

        :param metadata: The SiLA Client Metadata attached to the call
        :return: Optional `Queue` that should be used for updating this property.
            If None, the default Queue will be used.
        """
        pass

    @abstractmethod
    def StartDispensing(self, *, metadata: MetadataDict) -> StartDispensing_Responses:
        """
        Start dispensing of purified water. Depending on the device or the configured dispensing mode dispensing might be stopped automatically after a configured period of time (time-controlled mode), after a certain volume has been dosed (volume-controlled mode), or dispensing might not be stopped until StopDispensing is called ("manual" mode).


        :param metadata: The SiLA Client Metadata attached to the call

        """
        pass

    @abstractmethod
    def StopDispensing(self, *, metadata: MetadataDict) -> StopDispensing_Responses:
        """
        Stop dispensing of purified water. Depending on the device or the configured dispensing mode it might be possible to not only stop dispensing in "manual" mode but also in volume-controlled or time-controlled mode.


        :param metadata: The SiLA Client Metadata attached to the call

        """
        pass
