import datetime

from flask import Flask, jsonify
from werkzeug.exceptions import NotFound

app = Flask(__name__)

api_base_url = "/rest/v1_0"


@app.route(api_base_url + "/system/<path:subpath>", methods=["GET"])
def get_system(subpath: str):
    if subpath == "maintenance/reminder":
        return jsonify({"date": None, "alert": None})
    elif subpath == "quality_module/alert":
        return jsonify(None)
    elif subpath == "want_reset":
        return jsonify(False)
    else:
        raise NotFound()


@app.route(api_base_url + "/pre/ro/<path:subpath>", methods=["GET"])
def get_pre_ro(subpath: str):
    if subpath == "quality_in/conductivity":
        return jsonify(
            {
                "is_outdated": False,
                "maximum": None,
                "value": None,
                "alert": None,
                "step": 1,
                "minimum": None,
                "in_progress": False,
                "decimals": 0,
                "unit": "uS/cm c",
            }
        )
    elif subpath == "quality_in/temperature":
        return jsonify(
            {
                "is_outdated": False,
                "maximum": None,
                "value": None,
                "alert": None,
                "step": 1,
                "minimum": None,
                "in_progress": False,
                "decimals": 1,
                "unit": " C",
            }
        )
    elif subpath == "quality_out/conductivity":
        return jsonify(
            {
                "is_outdated": True,
                "maximum": None,
                "value": 6,
                "alert": None,
                "step": 1,
                "minimum": None,
                "in_progress": False,
                "decimals": 0,
                "unit": "uS/cm c",
            }
        )
    elif subpath == "quality_out/temperature":
        return jsonify(
            {
                "is_outdated": True,
                "maximum": None,
                "value": 24.4,
                "alert": None,
                "step": 1,
                "minimum": None,
                "in_progress": False,
                "decimals": 1,
                "unit": " C",
            }
        )
    elif subpath == "rejection_rate":
        return jsonify(
            {
                "is_outdated": False,
                "maximum": None,
                "value": None,
                "alert": None,
                "step": 1,
                "minimum": None,
                "in_progress": False,
                "decimals": 0,
                "unit": "%",
            }
        )
    else:
        raise NotFound()


@app.route(api_base_url + "/pre/<path:subpath>", methods=["GET"])
def get_pre(subpath: str):
    if subpath == "input_pressure":
        alert = {
            "token": 6,
            "category": "ERROR",
            "level": "MAJOR_ERROR",
            "issue": "LOW",
            "priority": 5,
            "limit": None,
            "acknowledged": True,
            "muted": True,
            "date": {"year": 2022, "month": 6, "day": 1},
            "time": {"hour": 15, "minute": 23, "second": 6},
        }
        return jsonify(
            {
                "is_outdated": False,
                "maximum": None,
                "value": None,
                # alert only on even minutes
                "alert": alert if datetime.datetime.now().minute % 2 == 0 else None,
                "step": 1,
                "minimum": None,
                "in_progress": False,
                "decimals": 3,
                "unit": "bar",
            }
        )
    elif subpath == "rejection_rate":
        return jsonify(
            {
                "is_outdated": False,
                "maximum": None,
                "value": None,
                "alert": None,
                "step": 1,
                "minimum": None,
                "in_progress": False,
                "decimals": 0,
                "unit": "%",
            }
        )
    elif subpath == "ro_module/reminder":
        return jsonify(
            {"date": {"month": 4, "day": 22, "year": 2024}, "consumable_type": "RO_MODULE_RO2", "alert": None}
        )
    elif subpath == "prefilter/reminder":
        return jsonify(
            # {"date": {"month": 8, "day": 1, "year": 2022}, "consumable_type": "PREFILTER_COMFORT", "alert": None}
            {
                "date": {"year": 2022, "month": 8, "day": 2},
                "consumable_type": "PREFILTER_COMFORT",
                "alert": {
                    "token": 30,
                    "category": "CARE",
                    "level": "WARNING",
                    "issue": "EXPIRED",
                    "priority": 60,
                    "limit": None,
                    "acknowledged": True,
                    "muted": True,
                    "date": {"year": 2022, "month": 8, "day": 2},
                    "time": {"hour": 7, "minute": 22, "second": 4},
                },
            }
        )
    elif subpath == "cleaning/reminder":
        return jsonify({"date": {"month": 8, "day": 31, "year": 2022}, "alert": None})
    else:
        raise NotFound()


@app.route(api_base_url + "/tank/bag/<path:subpath>", methods=["GET"])
def get_tank_bag(subpath: str):
    if subpath == "content":
        return jsonify(
            {
                "volume": {"step": 1, "decimals": 0, "unit": "l", "value": 20},
                "state": "MID",
                "alert": None,
                "factor": 1.0,
            }
        )
    elif subpath == "reminder":
        return jsonify({"date": {"month": 10, "day": 30, "year": 2022}, "consumable_type": "BAG_20", "alert": None})
    else:
        raise NotFound()


@app.route(api_base_url + "/tank/process/<path:subpath>", methods=["GET"])
def get_tank_process(subpath: str):
    if subpath == "is_output_possible":
        return jsonify(True)
    elif subpath == "volume_available":
        return jsonify({"step": 1, "decimals": 1, "unit": "l", "value": 17.8})
    else:
        raise NotFound()


@app.route(api_base_url + "/tank/process", methods=["GET"])
def get_tank():
    return jsonify(
        {
            "process": "DISPENSE",
            "product_type": "Arium Comfort",
            "state": "STOPPED",
            "last_run_issue": "NONE",
            "id": 0,
        }
    )


@app.route(api_base_url + "/final/process/<path:subpath>", methods=["GET"])
def get_final_process(subpath: str):
    if subpath == "is_output_possible":
        return jsonify(True)
    elif subpath == "volume_available":
        return jsonify({"step": 1, "decimals": 1, "unit": "l", "value": 17.8})
    else:
        raise NotFound()


@app.route(api_base_url + "/final/<path:subpath>", methods=["GET"])
def get_final(subpath: str):
    if subpath == "process":
        return jsonify(
            {
                "process": "DISPENSE",
                "product_type": "Arium Comfort",
                "state": "STOPPED",
                "last_run_issue": "NONE",
                "id": 0,
            }
        )
    elif subpath == "quality_out/conductivity":
        return jsonify(
            {
                "is_outdated": False,
                "maximum": None,
                "value": 0.055,
                "alert": None,
                "step": 1,
                "minimum": None,
                "in_progress": False,
                "decimals": 3,
                "unit": "uS/cm c",
            }
        )
    elif subpath == "quality_out/temperature":
        return jsonify(
            {
                "is_outdated": False,
                "maximum": None,
                "value": 28.4,
                "alert": None,
                "step": 1,
                "minimum": None,
                "in_progress": False,
                "decimals": 1,
                "unit": " C",
            }
        )
    elif subpath == "cartridge/reminder":
        return jsonify(
            {
                "date": {"month": 5, "day": 3, "year": 2023},
                "consumable_type": "FINALTREATMENT_CARTRIDGE_COMFORT",
                "alert": None,
            }
        )
    elif subpath == "uv_lamp/current":
        return jsonify(
            {
                "is_outdated": False,
                "maximum": None,
                "value": 0,
                "alert": None,
                "step": 1,
                "minimum": None,
                "in_progress": False,
                "decimals": 0,
                "unit": "mA",
            }
        )
    elif subpath == "uv_lamp/reminder":
        return jsonify({"date": {"month": 5, "day": 3, "year": 2023}, "consumable_type": "UV_LAMP_G2", "alert": None})
    elif subpath == "flow_sensor/alert":
        return jsonify(None)
    else:
        raise NotFound()


@app.route(api_base_url + "/dispense/final_filter/reminder", methods=["GET"])
def get_dispense_final_filter_reminder():
    return jsonify({"date": None, "consumable_type": "FINAL_FILTER_STERILE", "alert": None})


if __name__ == "__main__":
    app.run(debug=True)
